@@ -24,18 +24,28 @@
 
 import warnings
 from collections import OrderedDict
+from logging import getLogger
+import hashlib
 
 import numpy as np
 import yaml
+from pyproj import Geod, Proj
 
-from pyresample import _spatial_mp, utils
+from pyresample import _spatial_mp, utils, CHUNK_SIZE
+
+try:
+    from xarray import DataArray
+except ImportError:
+    DataArray = np.ndarray
+
+logger = getLogger(__name__)
 
 
-class DimensionError(Exception):
+class DimensionError(ValueError):
     pass
 
 
-class IncompatibleAreas(Exception):
+class IncompatibleAreas(ValueError):
     """Error when the areas to combine are not compatible."""
 
 
@@ -53,36 +63,31 @@ class Boundary(object):
 
 class BaseDefinition(object):
 
-    """Base class for geometry definitions"""
+    """Base class for geometry definitions
+
+    .. versionchanged:: 1.8.0
+
+        `BaseDefinition` no longer checks the validity of the provided
+        longitude and latitude coordinates to improve performance. Longitude
+        arrays are expected to be between -180 and 180 degrees, latitude -90
+        to 90 degrees. Use `pyresample.utils.check_and_wrap` to preprocess
+        your arrays.
+
+    """
 
     def __init__(self, lons=None, lats=None, nprocs=1):
         if type(lons) != type(lats):
             raise TypeError('lons and lats must be of same type')
         elif lons is not None:
+            if not isinstance(lons, (np.ndarray, DataArray)):
+                lons = np.asanyarray(lons)
+                lats = np.asanyarray(lats)
             if lons.shape != lats.shape:
                 raise ValueError('lons and lats must have same shape')
 
         self.nprocs = nprocs
-
-        # check the latitutes
-        if lats is not None and ((lats.min() < -90. or lats.max() > +90.)):
-            # throw exception
-            raise ValueError(
-                'Some latitudes are outside the [-90.;+90] validity range')
-        else:
-            self.lats = lats
-
-        # check the longitudes
-        if lons is not None and ((lons.min() < -180. or lons.max() >= +180.)):
-            # issue warning
-            warnings.warn('All geometry objects expect longitudes in the [-180:+180[ range. ' +
-                          'We will now automatically wrap your longitudes into [-180:+180[, and continue. ' +
-                          'To avoid this warning next time, use routine utils.wrap_longitudes().')
-            # wrap longitudes to [-180;+180[
-            self.lons = utils.wrap_longitudes(lons)
-        else:
-            self.lons = lons
-
+        self.lats = lats
+        self.lons = lons
         self.ndim = None
         self.cartesian_coords = None
 
@@ -115,26 +120,24 @@ class BaseDefinition(object):
         return not self.__eq__(other)
 
     def get_area_extent_for_subset(self, row_LR, col_LR, row_UL, col_UL):
-        """Retrieves area_extent for a subdomain
-        rows    are counted from upper left to lower left
-        columns are counted from upper left to upper right
+        """Calculate extent for a subdomain of this area
 
-        :Parameters:
-        row_LR : int
-            row of the lower right pixel
-        col_LR : int
-            col of the lower right pixel
-        row_UL : int
-            row of the upper left pixel
-        col_UL : int
-            col of the upper left pixel
+        Rows are counted from upper left to lower left and columns are
+        counted from upper left to upper right.
 
-        :Returns:
-        area_extent : list
-            Area extent as a list (LL_x, LL_y, UR_x, UR_y) of the subset
+        Args:
+            row_LR (int): row of the lower right pixel
+            col_LR (int): col of the lower right pixel
+            row_UL (int): row of the upper left pixel
+            col_UL (int): col of the upper left pixel
+
+        Returns:
+            area_extent (tuple):
+                Area extent (LL_x, LL_y, UR_x, UR_y) of the subset
+
+        Author:
+            Ulrich Hamann
 
-        :Author:
-        Ulrich Hamann
         """
 
         (a, b) = self.get_proj_coords(data_slice=(row_LR, col_LR))
@@ -144,7 +147,7 @@ class BaseDefinition(object):
         c = c + 0.5 * self.pixel_size_x
         d = d + 0.5 * self.pixel_size_y
 
-        return (a, b, c, d)
+        return a, b, c, d
 
     def get_lonlat(self, row, col):
         """Retrieve lon and lat of single pixel
@@ -183,7 +186,8 @@ class BaseDefinition(object):
         side2 = self.get_lonlats(data_slice=(slice(None), -1))
         side3 = self.get_lonlats(data_slice=(-1, slice(None)))
         side4 = self.get_lonlats(data_slice=(slice(None), 0))
-        return Boundary(side1[0], side2[0], side3[0][::-1], side4[0][::-1]), Boundary(side1[1], side2[1], side3[1][::-1], side4[1][::-1])
+        return (Boundary(side1[0], side2[0], side3[0][::-1], side4[0][::-1]),
+                Boundary(side1[1], side2[1], side3[1][::-1], side4[1][::-1]))
 
     def get_cartesian_coords(self, nprocs=None, data_slice=None, cache=False):
         """Retrieve cartesian coordinates of geometry definition
@@ -351,6 +355,9 @@ class CoordinateDefinition(BaseDefinition):
     """Base class for geometry definitions defined by lons and lats only"""
 
     def __init__(self, lons, lats, nprocs=1):
+        if not isinstance(lons, (np.ndarray, DataArray)):
+            lons = np.asanyarray(lons)
+            lats = np.asanyarray(lats)
         super(CoordinateDefinition, self).__init__(lons, lats, nprocs)
         if lons.shape == lats.shape and lons.dtype == lats.dtype:
             self.shape = lons.shape
@@ -421,9 +428,23 @@ class GridDefinition(CoordinateDefinition):
             raise ValueError('2 dimensional lon lat grid expected')
 
 
-class SwathDefinition(CoordinateDefinition):
+def get_array_hashable(arr):
+    """Compute a hashable form of the array `arr`.
 
-    """Swath defined by lons and lats
+    Works with numpy arrays, dask.array.Array, and xarray.DataArray.
+    """
+    # look for precomputed value
+    if isinstance(arr, DataArray) and np.ndarray is not DataArray:
+        return arr.attrs.get('hash', get_array_hashable(arr.data))
+    else:
+        try:
+            return arr.name.encode('utf-8')  # dask array
+        except AttributeError:
+            return np.asarray(arr).view(np.uint8)  # np array
+
+
+class SwathDefinition(CoordinateDefinition):
+    """Swath defined by lons and lats.
 
     Parameters
     ----------
@@ -446,15 +467,227 @@ class SwathDefinition(CoordinateDefinition):
         Swath lats
     cartesian_coords : object
         Swath cartesian coordinates
+
     """
 
     def __init__(self, lons, lats, nprocs=1):
+        if not isinstance(lons, (np.ndarray, DataArray)):
+            lons = np.asanyarray(lons)
+            lats = np.asanyarray(lats)
         super(SwathDefinition, self).__init__(lons, lats, nprocs)
         if lons.shape != lats.shape:
             raise ValueError('lon and lat arrays must have same shape')
         elif lons.ndim > 2:
             raise ValueError('Only 1 and 2 dimensional swaths are allowed')
 
+        self.hash = None
+
+    def __hash__(self):
+        """Compute the hash of this object."""
+        if self.hash is None:
+            hasher = hashlib.sha1()
+            hasher.update(get_array_hashable(self.lons))
+            hasher.update(get_array_hashable(self.lats))
+            try:
+                if self.lons.mask is not np.bool_(False):
+                    hasher.update(get_array_hashable(self.lons.mask))
+            except AttributeError:
+                pass
+            self.hash = int(hasher.hexdigest(), 16)
+
+        return self.hash
+
+    def get_lonlats_dask(self, chunks=CHUNK_SIZE):
+        """Get the lon lats as a single dask array."""
+        import dask.array as da
+        lons, lats = self.get_lonlats()
+
+        if isinstance(lons.data, da.Array):
+            return lons.data, lats.data
+        else:
+            lons = da.from_array(np.asanyarray(lons),
+                                 chunks=chunks)
+            lats = da.from_array(np.asanyarray(lats),
+                                 chunks=chunks)
+        return lons, lats
+
+    def _compute_omerc_parameters(self, ellipsoid):
+        """Compute the oblique mercator projection bouding box parameters."""
+        lines, cols = self.lons.shape
+        lon1, lon2 = np.asanyarray(self.lons[[0, -1], int(cols / 2)])
+        lat1, lat, lat2 = np.asanyarray(
+            self.lats[[0, int(lines / 2), -1], int(cols / 2)])
+
+        proj_dict2points = {'proj': 'omerc', 'lat_0': lat, 'ellps': ellipsoid,
+                            'lat_1': lat1, 'lon_1': lon1,
+                            'lat_2': lat2, 'lon_2': lon2}
+
+        lonc, lat0 = Proj(**proj_dict2points)(0, 0, inverse=True)
+        az1, az2, dist = Geod(**proj_dict2points).inv(lonc, lat0, lon1, lat1)
+        del az2, dist
+        return {'proj': 'omerc', 'alpha': float(az1),
+                'lat_0': float(lat0),  'lonc': float(lonc),
+                'no_rot': True, 'ellps': ellipsoid}
+
+    def _compute_generic_parameters(self, projection, ellipsoid):
+        """Compute the projection bb parameters for most projections."""
+        lines, cols = self.lons.shape
+        lat_0 = self.lats[int(lines / 2), int(cols / 2)]
+        lon_0 = self.lons[int(lines / 2), int(cols / 2)]
+        return {'proj': projection, 'ellps': ellipsoid,
+                'lat_0': lat_0, 'lon_0': lon_0}
+
+    def get_edge_lonlats(self):
+        """Get the concatenated boundary of the current swath."""
+        lons, lats = self.get_boundary_lonlats()
+        blons = np.ma.concatenate([lons.side1, lons.side2,
+                                   lons.side3, lons.side4])
+        blats = np.ma.concatenate([lats.side1, lats.side2,
+                                   lats.side3, lats.side4])
+        return blons, blats
+
+    def compute_bb_proj_params(self, proj_dict):
+        projection = proj_dict['proj']
+        ellipsoid = proj_dict.get('ellps', 'WGS84')
+        if projection == 'omerc':
+            return self._compute_omerc_parameters(ellipsoid)
+        else:
+            new_proj = self._compute_generic_parameters(projection, ellipsoid)
+            new_proj.update(proj_dict)
+            return new_proj
+
+    def compute_optimal_bb_area(self, proj_dict=None):
+        """Compute the "best" bounding box area for this swath with `proj_dict`.
+
+        By default, the projection is Oblique Mercator (`omerc` in proj.4), in
+        which case the right projection angle `alpha` is computed from the
+        swath centerline. For other projections, only the appropriate center of
+        projection and area extents are computed.
+        """
+        if proj_dict is None:
+            proj_dict = {}
+        projection = proj_dict.setdefault('proj', 'omerc')
+        area_id = projection + '_otf'
+        description = 'On-the-fly ' + projection + ' area'
+        lines, cols = self.lons.shape
+        x_size = int(cols * 1.1)
+        y_size = int(lines * 1.1)
+
+        proj_dict = self.compute_bb_proj_params(proj_dict)
+
+        if projection == 'omerc':
+            x_size, y_size = y_size, x_size
+
+        area = DynamicAreaDefinition(area_id, description, proj_dict)
+        lons, lats = self.get_edge_lonlats()
+        return area.freeze((lons, lats), size=(x_size, y_size))
+
+
+class DynamicAreaDefinition(object):
+    """An AreaDefintion containing just a subset of the needed parameters.
+
+    The purpose of this class is to be able to adapt the area extent and size
+    of the area to a given set of longitudes and latitudes, such that e.g.
+    polar satellite granules can be resampled optimaly to a give projection.
+    """
+
+    def __init__(self, area_id=None, description=None, proj_dict=None,
+                 x_size=None, y_size=None, area_extent=None,
+                 optimize_projection=False, rotation=None):
+        """Initialize the DynamicAreaDefinition.
+
+        area_id:
+          The name of the area.
+        description:
+          The description of the area.
+        proj_dict:
+          The dictionary of projection parameters. Doesn't have to be complete.
+        x_size, y_size:
+          The size of the resulting area.
+        area_extent:
+          The area extent of the area.
+        optimize_projection:
+          Whether the projection parameters have to be optimized.
+        rotation:
+          Rotation in degrees (negative is cw)
+          """
+        self.area_id = area_id
+        self.description = description
+        self.proj_dict = proj_dict
+        self.x_size = x_size
+        self.y_size = y_size
+        self.area_extent = area_extent
+        self.optimize_projection = optimize_projection
+        self.rotation = rotation
+
+    def compute_domain(self, corners, resolution=None, size=None):
+        """Compute size and area_extent from corners and [size or resolution]
+        info."""
+        if resolution is not None and size is not None:
+            raise ValueError("Both resolution and size can't be provided.")
+
+        if size:
+            x_size, y_size = size
+            x_resolution = (corners[2] - corners[0]) * 1.0 / (x_size - 1)
+            y_resolution = (corners[3] - corners[1]) * 1.0 / (y_size - 1)
+
+        if resolution:
+            try:
+                x_resolution, y_resolution = resolution
+            except TypeError:
+                x_resolution = y_resolution = resolution
+            x_size = int(np.rint((corners[2] - corners[0]) * 1.0 /
+                                 x_resolution + 1))
+            y_size = int(np.rint((corners[3] - corners[1]) * 1.0 /
+                                 y_resolution + 1))
+
+        area_extent = (corners[0] - x_resolution / 2,
+                       corners[1] - y_resolution / 2,
+                       corners[2] + x_resolution / 2,
+                       corners[3] + y_resolution / 2)
+        return area_extent, x_size, y_size
+
+    def freeze(self, lonslats=None,
+               resolution=None, size=None,
+               proj_info=None, rotation=None):
+        """Create an AreaDefintion from this area with help of some extra info.
+
+        lonlats:
+          the geographical coordinates to contain in the resulting area.
+        resolution:
+          the resolution of the resulting area.
+        size:
+          the size of the resulting area.
+        proj_info:
+          complementing parameters to the projection info.
+        rotation:
+          rotation in degrees (negative is cw)
+
+        Resolution and Size parameters are ignored if the instance is created
+        with the `optimize_projection` flag set to True.
+        """
+        if proj_info is not None:
+            self.proj_dict.update(proj_info)
+
+        if self.optimize_projection:
+            return lonslats.compute_optimal_bb_area(self.proj_dict)
+
+        if not self.area_extent or not self.x_size or not self.y_size:
+            proj4 = Proj(**self.proj_dict)
+            try:
+                lons, lats = lonslats
+            except (TypeError, ValueError):
+                lons, lats = lonslats.get_lonlats()
+            xarr, yarr = proj4(np.asarray(lons), np.asarray(lats))
+            corners = [np.min(xarr), np.min(yarr), np.max(xarr), np.max(yarr)]
+
+            domain = self.compute_domain(corners, resolution, size)
+            self.area_extent, self.x_size, self.y_size = domain
+
+        return AreaDefinition(self.area_id, self.description, '',
+                              self.proj_dict, self.x_size, self.y_size,
+                              self.area_extent, self.rotation)
+
 
 class AreaDefinition(BaseDefinition):
 
@@ -474,6 +707,8 @@ class AreaDefinition(BaseDefinition):
         x dimension in number of pixels
     y_size : int
         y dimension in number of pixels
+    rotation: float
+        rotation in degrees (negative is cw)
     area_extent : list
         Area extent as a list (LL_x, LL_y, UR_x, UR_y)
     nprocs : int, optional
@@ -497,6 +732,8 @@ class AreaDefinition(BaseDefinition):
         x dimension in number of pixels
     y_size : int
         y dimension in number of pixels
+    rotation: float
+        rotation in degrees (negative is cw)
     shape : tuple
         Corresponding array shape as (rows, cols)
     size : int
@@ -532,7 +769,8 @@ class AreaDefinition(BaseDefinition):
     """
 
     def __init__(self, area_id, name, proj_id, proj_dict, x_size, y_size,
-                 area_extent, nprocs=1, lons=None, lats=None, dtype=np.float64):
+                 area_extent, rotation=None, nprocs=1, lons=None, lats=None,
+                 dtype=np.float64):
         if not isinstance(proj_dict, dict):
             raise TypeError('Wrong type for proj_dict: %s. Expected dict.'
                             % type(proj_dict))
@@ -541,9 +779,13 @@ class AreaDefinition(BaseDefinition):
         self.area_id = area_id
         self.name = name
         self.proj_id = proj_id
-        self.x_size = x_size
-        self.y_size = y_size
+        self.x_size = int(x_size)
+        self.y_size = int(y_size)
         self.shape = (y_size, x_size)
+        try:
+            self.rotation = float(rotation)
+        except TypeError:
+            self.rotation = 0
         if lons is not None:
             if lons.shape != self.shape:
                 raise ValueError('Shape of lon lat grid must match '
@@ -570,16 +812,20 @@ class AreaDefinition(BaseDefinition):
              float(area_extent[3]) -
              float(self.pixel_size_y) / 2)
 
-        # Pixel_offset defines the distance to projection center from origen (UL)
-        # of image in units of pixels.
+        # Pixel_offset defines the distance to projection center from origen
+        # (UL) of image in units of pixels.
         self.pixel_offset_x = -self.area_extent[0] / self.pixel_size_x
         self.pixel_offset_y = self.area_extent[3] / self.pixel_size_y
 
-        self.projection_x_coords = None
-        self.projection_y_coords = None
+        self._projection_x_coords = None
+        self._projection_y_coords = None
 
         self.dtype = dtype
 
+    @property
+    def proj_str(self):
+        return utils.proj4_dict_to_str(self.proj_dict, sort=True)
+
     def __str__(self):
         # We need a sorted dictionary for a unique hash of str(self)
         proj_dict = self.proj_dict
@@ -588,7 +834,7 @@ class AreaDefinition(BaseDefinition):
                                for k in sorted(proj_dict.keys())]) +
                     '}')
 
-        if self.proj_id is None:
+        if not self.proj_id:
             third_line = ""
         else:
             third_line = "Projection ID: {0}\n".format(self.proj_id)
@@ -627,6 +873,7 @@ class AreaDefinition(BaseDefinition):
         fmt += "\tPCS_DEF:\t{proj_str}\n"
         fmt += "\tXSIZE:\t{x_size}\n"
         fmt += "\tYSIZE:\t{y_size}\n"
+        fmt += "\tROTATION:\t{rotation}\n"
         fmt += "\tAREA_EXTENT: {area_extent}\n}};\n"
         area_def_str = fmt.format(name=self.name, area_id=self.area_id,
                                   proj_str=proj_str, x_size=self.x_size,
@@ -640,7 +887,7 @@ class AreaDefinition(BaseDefinition):
         """Test for equality"""
 
         try:
-            return ((self.proj_dict == other.proj_dict) and
+            return ((self.proj_str == other.proj_str) and
                     (self.shape == other.shape) and
                     (np.allclose(self.area_extent, other.area_extent)))
         except AttributeError:
@@ -651,6 +898,13 @@ class AreaDefinition(BaseDefinition):
 
         return not self.__eq__(other)
 
+    def __hash__(self):
+        return hash((
+            self.proj_str,
+            self.shape,
+            self.area_extent
+        ))
+
     def colrow2lonlat(self, cols, rows):
         """
         Return longitudes and latitudes for the given image columns
@@ -659,8 +913,8 @@ class AreaDefinition(BaseDefinition):
         (see get_lonlats).
         """
         p = _spatial_mp.Proj(self.proj4_string)
-        x = self.proj_x_coords
-        y = self.proj_y_coords
+        x = self.projection_x_coords
+        y = self.projection_y_coords
         return p(y[y.size - cols], x[x.size - rows], inverse=True)
 
     def lonlat2colrow(self, lons, lats):
@@ -709,37 +963,42 @@ class AreaDefinition(BaseDefinition):
 
         return self.get_xy_from_proj_coords(xm_, ym_)
 
-    def get_xy_from_proj_coords(self, xm_, ym_):
-        """Retrieve closest x and y coordinates (column, row indices) for a
-        location specified with projection coordinates (xm_,ym_) in meters.
-        A ValueError is raised, if the return point is outside the area domain. If
-        xm_,ym_ is a tuple of sequences of projection coordinates, a tuple of
-        masked arrays are returned.
+    def get_xy_from_proj_coords(self, xm, ym):
+        """Find closest grid cell index for a specified projection coordinate.
 
-        :Input:
-        xm_ : point or sequence (list or array) of x-coordinates in m (map projection)
-        ym_ : point or sequence (list or array) of y-coordinates in m (map projection)
+        If xm, ym is a tuple of sequences of projection coordinates, a tuple
+        of masked arrays are returned.
+
+        Args:
+            xm (list or array): point or sequence of x-coordinates in
+                                 meters (map projection)
+            ym (list or array): point or sequence of y-coordinates in
+                                 meters (map projection)
+
+        Returns:
+            x, y : column and row grid cell indexes as 2 scalars or arrays
+
+        Raises:
+            ValueError: if the return point is outside the area domain
 
-        :Returns:
-        (x, y) : tuple of integer points/arrays
         """
 
-        if isinstance(xm_, list):
-            xm_ = np.array(xm_)
-        if isinstance(ym_, list):
-            ym_ = np.array(ym_)
+        if isinstance(xm, list):
+            xm = np.array(xm)
+        if isinstance(ym, list):
+            ym = np.array(ym)
 
-        if ((isinstance(xm_, np.ndarray) and
-             not isinstance(ym_, np.ndarray)) or
-            (not isinstance(xm_, np.ndarray) and
-             isinstance(ym_, np.ndarray))):
-            raise ValueError("Both projection coordinates xm_ and ym_ needs to be of " +
+        if ((isinstance(xm, np.ndarray) and
+             not isinstance(ym, np.ndarray)) or
+            (not isinstance(xm, np.ndarray) and
+             isinstance(ym, np.ndarray))):
+            raise ValueError("Both projection coordinates xm and ym needs to be of " +
                              "the same type and have the same dimensions!")
 
-        if isinstance(xm_, np.ndarray) and isinstance(ym_, np.ndarray):
-            if xm_.shape != ym_.shape:
+        if isinstance(xm, np.ndarray) and isinstance(ym, np.ndarray):
+            if xm.shape != ym.shape:
                 raise ValueError(
-                    "projection coordinates xm_ and ym_ is not of the same shape!")
+                    "projection coordinates xm and ym is not of the same shape!")
 
         upl_x = self.area_extent[0]
         upl_y = self.area_extent[3]
@@ -749,8 +1008,8 @@ class AreaDefinition(BaseDefinition):
         yscale = (self.area_extent[1] -
                   self.area_extent[3]) / float(self.y_size)
 
-        x__ = (xm_ - upl_x) / xscale
-        y__ = (ym_ - upl_y) / yscale
+        x__ = (xm - upl_x) / xscale
+        y__ = (ym - upl_y) / yscale
 
         if isinstance(x__, np.ndarray) and isinstance(y__, np.ndarray):
             mask = (((x__ < 0) | (x__ > self.x_size)) |
@@ -780,21 +1039,45 @@ class AreaDefinition(BaseDefinition):
 
         return self.get_lonlats(nprocs=None, data_slice=(row, col))
 
+    def get_proj_vectors_dask(self, chunks=CHUNK_SIZE, dtype=None):
+        import dask.array as da
+        if dtype is None:
+            dtype = self.dtype
+
+        target_x = da.arange(self.x_size, chunks=chunks, dtype=dtype) * \
+            self.pixel_size_x + self.pixel_upper_left[0]
+        target_y = da.arange(self.y_size, chunks=chunks, dtype=dtype) * - \
+            self.pixel_size_y + self.pixel_upper_left[1]
+        return target_x, target_y
+
+    def get_proj_coords_dask(self, chunks=CHUNK_SIZE, dtype=None):
+        # TODO: Add rotation
+        import dask.array as da
+        target_x, target_y = self.get_proj_vectors_dask(chunks, dtype)
+        return da.meshgrid(target_x, target_y)
+
     def get_proj_coords(self, data_slice=None, cache=False, dtype=None):
-        """Get projection coordinates of grid
+        """Get projection coordinates of grid.
 
         Parameters
         ----------
         data_slice : slice object, optional
             Calculate only coordinates for specified slice
         cache : bool, optional
-            Store result the result. Requires data_slice to be None
+            Store the result. Requires data_slice to be None
 
         Returns
         -------
         (target_x, target_y) : tuple of numpy arrays
             Grids of area x- and y-coordinates in projection units
+
         """
+        def do_rotation(xspan, yspan, rot_deg=0):
+            rot_rad = np.radians(rot_deg)
+            rot_mat = np.array([[np.cos(rot_rad),  np.sin(rot_rad)],
+                                [-np.sin(rot_rad), np.cos(rot_rad)]])
+            x, y = np.meshgrid(xspan, yspan)
+            return np.einsum('ji, mni -> jmn', rot_mat, np.dstack([x, y]))
 
         def get_val(val, sub_val, max):
             # Get value with substitution and wrapping
@@ -807,12 +1090,12 @@ class AreaDefinition(BaseDefinition):
                 else:
                     return val
 
-        if self.projection_x_coords is not None and self.projection_y_coords is not None:
+        if self._projection_x_coords is not None and self._projection_y_coords is not None:
             # Projection coords are cached
             if data_slice is None:
-                return self.projection_x_coords, self.projection_y_coords
+                return self._projection_x_coords, self._projection_y_coords
             else:
-                return self.projection_x_coords[data_slice], self.projection_y_coords[data_slice]
+                return self._projection_x_coords[data_slice], self._projection_y_coords[data_slice]
 
         is_single_value = False
         is_1d_select = False
@@ -870,17 +1153,15 @@ class AreaDefinition(BaseDefinition):
                 cols = 1
 
         # Calculate coordinates
-        target_x = np.fromfunction(lambda i, j: (j + col_start) *
-                                   self.pixel_size_x +
-                                   self.pixel_upper_left[0],
-                                   (rows,
-                                    cols), dtype=dtype)
-
-        target_y = np.fromfunction(lambda i, j:
-                                   self.pixel_upper_left[1] -
-                                   (i + row_start) * self.pixel_size_y,
-                                   (rows,
-                                    cols), dtype=dtype)
+        target_x = np.arange(col_start, col_start + cols, dtype=dtype) * \
+            self.pixel_size_x + self.pixel_upper_left[0]
+        target_y = np.arange(row_start, row_start + rows, dtype=dtype) * - \
+            self.pixel_size_y + self.pixel_upper_left[1]
+        if self.rotation != 0:
+            res = do_rotation(target_x, target_y, self.rotation)
+            target_x, target_y = res[0, :, :], res[1, :, :]
+        else:
+            target_x, target_y = np.meshgrid(target_x, target_y)
 
         if is_single_value:
             # Return single values
@@ -893,19 +1174,31 @@ class AreaDefinition(BaseDefinition):
 
         if cache and data_slice is None:
             # Cache the result if requested
-            self.projection_x_coords = target_x
-            self.projection_y_coords = target_y
+            self._projection_x_coords = target_x
+            self._projection_y_coords = target_y
 
         return target_x, target_y
 
     @property
-    def proj_x_coords(self):
+    def projection_x_coords(self):
         return self.get_proj_coords(data_slice=(0, slice(None)))[0]
 
     @property
-    def proj_y_coords(self):
+    def projection_y_coords(self):
         return self.get_proj_coords(data_slice=(slice(None), 0))[1]
 
+    @property
+    def proj_x_coords(self):
+        warnings.warn(
+            "Deprecated, use 'projection_x_coords' instead", DeprecationWarning)
+        return self.projection_x_coords
+
+    @property
+    def proj_y_coords(self):
+        warnings.warn(
+            "Deprecated, use 'projection_y_coords' instead", DeprecationWarning)
+        return self.projection_y_coords
+
     @property
     def outer_boundary_corners(self):
         """Returns the lon,lat of the outer edges of the corner points
@@ -923,6 +1216,24 @@ class AreaDefinition(BaseDefinition):
                 Coordinate(corner_lons[2], corner_lats[2]),
                 Coordinate(corner_lons[3], corner_lats[3])]
 
+    def get_lonlats_dask(self, chunks=CHUNK_SIZE, dtype=None):
+        from dask.array import map_blocks
+
+        dtype = dtype or self.dtype
+        target_x, target_y = self.get_proj_coords_dask(chunks, dtype)
+
+        target_proj = Proj(**self.proj_dict)
+
+        def invproj(data1, data2):
+            return np.dstack(target_proj(data1, data2, inverse=True))
+
+        res = map_blocks(invproj, target_x, target_y, chunks=(target_x.chunks[0],
+                                                              target_x.chunks[1],
+                                                              2),
+                         new_axis=[2])
+
+        return res[:, :, 0], res[:, :, 1]
+
     def get_lonlats(self, nprocs=None, data_slice=None, cache=False, dtype=None):
         """Returns lon and lat arrays of area.
 
@@ -1026,8 +1337,8 @@ def concatenate_area_defs(area1, area2, axis=0):
 
     if axis == 0:
         area_extent = combine_area_extents_vertical(area1, area2)
-        x_size = area1.x_size
-        y_size = area1.y_size + area2.y_size
+        x_size = int(area1.x_size)
+        y_size = int(area1.y_size + area2.y_size)
     else:
         raise NotImplementedError('Only vertical contatenation is supported.')
     return AreaDefinition(area1.area_id, area1.name, area1.proj_id,
@@ -1112,6 +1423,23 @@ class StackedAreaDefinition(BaseDefinition):
 
         return self.lons, self.lats
 
+    def get_lonlats_dask(self, chunks=CHUNK_SIZE, dtype=None):
+        """"Return lon and lat dask arrays of the area."""
+        import dask.array as da
+        llons = []
+        llats = []
+        for definition in self.defs:
+            lons, lats = definition.get_lonlats_dask(chunks=chunks,
+                                                     dtype=dtype)
+
+            llons.append(lons)
+            llats.append(lats)
+
+        self.lons = da.concatenate(llons, axis=0)
+        self.lats = da.concatenate(llats, axis=0)
+
+        return self.lons, self.lats
+
     def squeeze(self):
         """Generate a single AreaDefinition if possible."""
         if len(self.defs) == 1:
