@@ -4,7 +4,6 @@ import random
 import sys
 
 import numpy as np
-from mock import MagicMock, patch
 
 from pyresample import geo_filter, geometry
 from pyresample.geometry import (IncompatibleAreas,
@@ -12,6 +11,12 @@ from pyresample.geometry import (IncompatibleAreas,
                                  concatenate_area_defs)
 from pyresample.test.utils import catch_warnings
 
+try:
+    from unittest.mock import MagicMock, patch
+except ImportError:
+    # separate mock package py<3.3
+    from mock import MagicMock, patch
+
 if sys.version_info < (2, 7):
     import unittest2 as unittest
 else:
@@ -86,43 +91,6 @@ class Test(unittest.TestCase):
         self.assertTrue((cart_coords.sum() - exp) < 1e-7 * exp,
                         msg='Calculation of cartesian coordinates failed')
 
-    def test_base_lat_invalid(self):
-
-        lons = np.arange(-135., +135, 20.)
-        lats = np.ones_like(lons) * 70.
-        lats[0] = -95
-        lats[1] = +95
-        self.assertRaises(
-            ValueError, geometry.BaseDefinition, lons=lons, lats=lats)
-
-    def test_base_lon_wrapping(self):
-
-        lons1 = np.arange(-135., +135, 50.)
-        lats = np.ones_like(lons1) * 70.
-
-        with catch_warnings() as w1:
-            base_def1 = geometry.BaseDefinition(lons1, lats)
-            self.assertFalse(
-                len(w1) != 0, 'Got warning <%s>, but was not expecting one' % w1)
-
-        lons2 = np.where(lons1 < 0, lons1 + 360, lons1)
-        with catch_warnings() as w2:
-            base_def2 = geometry.BaseDefinition(lons2, lats)
-            self.assertFalse(
-                len(w2) != 1, 'Failed to trigger a warning on longitude wrapping')
-            self.assertFalse(('-180:+180' not in str(w2[0].message)),
-                             'Failed to trigger correct warning about longitude wrapping')
-
-        self.assertFalse(
-            base_def1 != base_def2, 'longitude wrapping to [-180:+180] did not work')
-
-        with catch_warnings() as w3:
-            base_def3 = geometry.BaseDefinition(None, None)
-            self.assertFalse(
-                len(w3) != 0, 'Got warning <%s>, but was not expecting one' % w3)
-
-        self.assert_raises(ValueError, base_def3.get_lonlats)
-
     def test_base_type(self):
         lons1 = np.arange(-135., +135, 50.)
         lats = np.ones_like(lons1) * 70.
@@ -160,38 +128,139 @@ class Test(unittest.TestCase):
                          "BaseDefinition did not maintain dtype of longitudes (in:%s out:%s)" %
                          (lons2_ints.dtype, lons.dtype,))
 
-    def test_swath(self):
-        lons1 = np.fromfunction(lambda y, x: 3 + (10.0 / 100) * x, (5000, 100))
-        lats1 = np.fromfunction(
-            lambda y, x: 75 - (50.0 / 5000) * y, (5000, 100))
+    def test_area_hash(self):
+        area_def = geometry.AreaDefinition('areaD', 'Europe (3km, HRV, VTC)', 'areaD',
+                                           {'a': '6378144.0',
+                                            'b': '6356759.0',
+                                            'lat_0': '50.00',
+                                            'lat_ts': '50.00',
+                                            'lon_0': '8.00',
+                                            'proj': 'stere'},
+                                           800,
+                                           800,
+                                           [-1370912.72,
+                                               -909968.64000000001,
+                                               1029087.28,
+                                               1490031.3600000001])
 
-        swath_def = geometry.SwathDefinition(lons1, lats1)
+        self.assertIsInstance(hash(area_def), int)
 
-        lons2, lats2 = swath_def.get_lonlats()
+        area_def = geometry.AreaDefinition('areaD', 'Europe (3km, HRV, VTC)', 'areaD',
+                                           {'a': '6378144.0',
+                                            'b': '6356759.0',
+                                            'lat_ts': '50.00',
+                                            'lon_0': '8.00',
+                                            'lat_0': '50.00',
+                                            'proj': 'stere'},
+                                           800,
+                                           800,
+                                           [-1370912.72,
+                                               -909968.64000000001,
+                                               1029087.28,
+                                               1490031.3600000001])
 
-        self.assertFalse(id(lons1) != id(lons2) or id(lats1) != id(lats2),
-                         msg='Caching of swath coordinates failed')
+        self.assertIsInstance(hash(area_def), int)
 
-    def test_swath_wrap(self):
-        lons1 = np.fromfunction(lambda y, x: 3 + (10.0 / 100) * x, (5000, 100))
-        lats1 = np.fromfunction(
-            lambda y, x: 75 - (50.0 / 5000) * y, (5000, 100))
+        area_def = geometry.AreaDefinition('New area', 'Europe', 'areaD',
+                                           {'a': '6378144.0',
+                                            'b': '6356759.0',
+                                            'lat_ts': '50.00',
+                                            'lon_0': '8.00',
+                                            'lat_0': '50.00',
+                                            'proj': 'stere'},
+                                           800,
+                                           800,
+                                           [-1370912.72,
+                                               -909968.64000000001,
+                                               1029087.28,
+                                               1490031.3600000001])
 
-        lons1 += 180.
-        with catch_warnings() as w1:
-            swath_def = geometry.BaseDefinition(lons1, lats1)
-            self.assertFalse(
-                len(w1) != 1, 'Failed to trigger a warning on longitude wrapping')
-            self.assertFalse(('-180:+180' not in str(w1[0].message)),
-                             'Failed to trigger correct warning about longitude wrapping')
+        self.assertIsInstance(hash(area_def), int)
 
-        lons2, lats2 = swath_def.get_lonlats()
+    def test_get_array_hashable(self):
+        arr = np.array([1.2, 1.3, 1.4, 1.5])
+        if sys.byteorder == 'little':
+            # arr.view(np.uint8)
+            reference = np.array([ 51,  51,  51,  51,  51,  51, 243,
+                                   63, 205, 204, 204, 204, 204,
+                                   204, 244,  63, 102, 102, 102, 102,
+                                   102, 102, 246,  63,   0,   0,
+                                   0,   0,   0,   0, 248,  63],
+                                   dtype=np.uint8)
+        else:
+            # on le machines use arr.byteswap().view(np.uint8)
+            reference = np.array([ 63, 243,  51,  51,  51,  51,  51,
+                                   51,  63, 244, 204, 204, 204,
+                                   204, 204, 205,  63, 246, 102, 102,
+                                   102, 102, 102, 102,  63, 248,
+                                   0,   0,   0,   0,   0,   0],
+                                   dtype=np.uint8)
 
-        self.assertTrue(id(lons1) != id(lons2),
-                        msg='Caching of swath coordinates failed with longitude wrapping')
+        np.testing.assert_allclose(reference,
+                                    geometry.get_array_hashable(arr))
+
+        try:
+            import xarray as xr
+        except ImportError:
+            pass
+        else:
+            xrarr = xr.DataArray(arr)
+            np.testing.assert_allclose(reference,
+                                       geometry.get_array_hashable(arr))
+
+            xrarr.attrs['hash'] = 42
+            self.assertEqual(geometry.get_array_hashable(xrarr),
+                             xrarr.attrs['hash'])
+
+
+    def test_swath_hash(self):
+        lons = np.array([1.2, 1.3, 1.4, 1.5])
+        lats = np.array([65.9, 65.86, 65.82, 65.78])
+        swath_def = geometry.SwathDefinition(lons, lats)
+
+        self.assertIsInstance(hash(swath_def), int)
+
+        try:
+            import dask.array as da
+        except ImportError:
+            print("Not testing with dask arrays")
+        else:
+            dalons = da.from_array(lons, chunks=1000)
+            dalats = da.from_array(lats, chunks=1000)
+            swath_def = geometry.SwathDefinition(dalons, dalats)
+
+            self.assertIsInstance(hash(swath_def), int)
+
+        try:
+            import xarray as xr
+        except ImportError:
+            print("Not testing with xarray")
+        else:
+            xrlons = xr.DataArray(lons)
+            xrlats = xr.DataArray(lats)
+            swath_def = geometry.SwathDefinition(xrlons, xrlats)
+
+            self.assertIsInstance(hash(swath_def), int)
+
+        try:
+            import xarray as xr
+            import dask.array as da
+        except ImportError:
+            print("Not testing with xarrays and dask arrays")
+        else:
+            xrlons = xr.DataArray(da.from_array(lons, chunks=1000))
+            xrlats = xr.DataArray(da.from_array(lats, chunks=1000))
+            swath_def = geometry.SwathDefinition(xrlons, xrlats)
+
+            self.assertIsInstance(hash(swath_def), int)
+
+
+        lons = np.ma.array([1.2, 1.3, 1.4, 1.5])
+        lats = np.ma.array([65.9, 65.86, 65.82, 65.78])
+        swath_def = geometry.SwathDefinition(lons, lats)
+
+        self.assertIsInstance(hash(swath_def), int)
 
-        self.assertTrue(lons2.min() > -180 and lons2.max() < 180,
-                        'Wrapping of longitudes failed for SwathDefinition')
 
     def test_area_equal(self):
         area_def = geometry.AreaDefinition('areaD', 'Europe (3km, HRV, VTC)', 'areaD',
@@ -255,23 +324,6 @@ class Test(unittest.TestCase):
         self.assertFalse(
             area_def == msg_area, 'area_defs are not expected to be equal')
 
-    def test_swath_equal(self):
-        lons = np.array([1.2, 1.3, 1.4, 1.5])
-        lats = np.array([65.9, 65.86, 65.82, 65.78])
-        swath_def = geometry.SwathDefinition(lons, lats)
-        swath_def2 = geometry.SwathDefinition(lons, lats)
-        self.assertFalse(
-            swath_def != swath_def2, 'swath_defs are not equal as expected')
-
-    def test_swath_not_equal(self):
-        lats1 = np.array([65.9, 65.86, 65.82, 65.78])
-        lons = np.array([1.2, 1.3, 1.4, 1.5])
-        lats2 = np.array([65.91, 65.85, 65.80, 65.75])
-        swath_def = geometry.SwathDefinition(lons, lats1)
-        swath_def2 = geometry.SwathDefinition(lons, lats2)
-        self.assertFalse(
-            swath_def == swath_def2, 'swath_defs are not expected to be equal')
-
     def test_swath_equal_area(self):
         area_def = geometry.AreaDefinition('areaD', 'Europe (3km, HRV, VTC)', 'areaD',
                                            {'a': '6378144.0',
@@ -348,60 +400,6 @@ class Test(unittest.TestCase):
         self.assertFalse(
             area_def == swath_def, "swath_def and area_def should be different")
 
-    def test_concat_1d(self):
-        lons1 = np.array([1, 2, 3])
-        lats1 = np.array([1, 2, 3])
-        lons2 = np.array([4, 5, 6])
-        lats2 = np.array([4, 5, 6])
-        swath_def1 = geometry.SwathDefinition(lons1, lats1)
-        swath_def2 = geometry.SwathDefinition(lons2, lats2)
-        swath_def_concat = swath_def1.concatenate(swath_def2)
-        expected = np.array([1, 2, 3, 4, 5, 6])
-        self.assertTrue(np.array_equal(swath_def_concat.lons, expected) and
-                        np.array_equal(swath_def_concat.lons, expected),
-                        'Failed to concatenate 1D swaths')
-
-    def test_concat_2d(self):
-        lons1 = np.array([[1, 2, 3], [3, 4, 5], [5, 6, 7]])
-        lats1 = np.array([[1, 2, 3], [3, 4, 5], [5, 6, 7]])
-        lons2 = np.array([[4, 5, 6], [6, 7, 8]])
-        lats2 = np.array([[4, 5, 6], [6, 7, 8]])
-        swath_def1 = geometry.SwathDefinition(lons1, lats1)
-        swath_def2 = geometry.SwathDefinition(lons2, lats2)
-        swath_def_concat = swath_def1.concatenate(swath_def2)
-        expected = np.array(
-            [[1, 2, 3], [3, 4, 5], [5, 6, 7], [4, 5, 6], [6, 7, 8]])
-        self.assertTrue(np.array_equal(swath_def_concat.lons, expected) and
-                        np.array_equal(swath_def_concat.lons, expected),
-                        'Failed to concatenate 2D swaths')
-
-    def test_append_1d(self):
-        lons1 = np.array([1, 2, 3])
-        lats1 = np.array([1, 2, 3])
-        lons2 = np.array([4, 5, 6])
-        lats2 = np.array([4, 5, 6])
-        swath_def1 = geometry.SwathDefinition(lons1, lats1)
-        swath_def2 = geometry.SwathDefinition(lons2, lats2)
-        swath_def1.append(swath_def2)
-        expected = np.array([1, 2, 3, 4, 5, 6])
-        self.assertTrue(np.array_equal(swath_def1.lons, expected) and
-                        np.array_equal(swath_def1.lons, expected),
-                        'Failed to append 1D swaths')
-
-    def test_append_2d(self):
-        lons1 = np.array([[1, 2, 3], [3, 4, 5], [5, 6, 7]])
-        lats1 = np.array([[1, 2, 3], [3, 4, 5], [5, 6, 7]])
-        lons2 = np.array([[4, 5, 6], [6, 7, 8]])
-        lats2 = np.array([[4, 5, 6], [6, 7, 8]])
-        swath_def1 = geometry.SwathDefinition(lons1, lats1)
-        swath_def2 = geometry.SwathDefinition(lons2, lats2)
-        swath_def1.append(swath_def2)
-        expected = np.array(
-            [[1, 2, 3], [3, 4, 5], [5, 6, 7], [4, 5, 6], [6, 7, 8]])
-        self.assertTrue(np.array_equal(swath_def1.lons, expected) and
-                        np.array_equal(swath_def1.lons, expected),
-                        'Failed to append 2D swaths')
-
     def test_grid_filter_valid(self):
         lons = np.array([-170, -30, 30, 170])
         lats = np.array([20, -40, 50, -80])
@@ -505,7 +503,7 @@ class Test(unittest.TestCase):
                                                -909968.64000000001,
                                                1029087.28,
                                                1490031.3600000001])
-        proj_x_boundary, proj_y_boundary = area_def.proj_x_coords, area_def.proj_y_coords
+        proj_x_boundary, proj_y_boundary = area_def.projection_x_coords, area_def.projection_y_coords
         expected_x = np.array([-1250912.72, -1010912.72, -770912.72,
                                -530912.72, -290912.72, -50912.72, 189087.28,
                                429087.28, 669087.28, 909087.28])
@@ -701,6 +699,184 @@ class Test(unittest.TestCase):
         self.assertTrue((y__.data == y_expects).all())
 
 
+def assert_np_dict_allclose(dict1, dict2):
+
+    assert set(dict1.keys()) == set(dict2.keys())
+    for key, val in dict1.items():
+        try:
+            np.testing.assert_allclose(val, dict2[key])
+        except TypeError:
+            assert(val == dict2[key])
+
+
+class TestSwathDefinition(unittest.TestCase):
+    """Test the SwathDefinition."""
+
+    def test_swath(self):
+        lons1 = np.fromfunction(lambda y, x: 3 + (10.0 / 100) * x, (5000, 100))
+        lats1 = np.fromfunction(
+            lambda y, x: 75 - (50.0 / 5000) * y, (5000, 100))
+
+        swath_def = geometry.SwathDefinition(lons1, lats1)
+
+        lons2, lats2 = swath_def.get_lonlats()
+
+        self.assertFalse(id(lons1) != id(lons2) or id(lats1) != id(lats2),
+                         msg='Caching of swath coordinates failed')
+
+    def test_concat_1d(self):
+        lons1 = np.array([1, 2, 3])
+        lats1 = np.array([1, 2, 3])
+        lons2 = np.array([4, 5, 6])
+        lats2 = np.array([4, 5, 6])
+        swath_def1 = geometry.SwathDefinition(lons1, lats1)
+        swath_def2 = geometry.SwathDefinition(lons2, lats2)
+        swath_def_concat = swath_def1.concatenate(swath_def2)
+        expected = np.array([1, 2, 3, 4, 5, 6])
+        self.assertTrue(np.array_equal(swath_def_concat.lons, expected) and
+                        np.array_equal(swath_def_concat.lons, expected),
+                        'Failed to concatenate 1D swaths')
+
+    def test_concat_2d(self):
+        lons1 = np.array([[1, 2, 3], [3, 4, 5], [5, 6, 7]])
+        lats1 = np.array([[1, 2, 3], [3, 4, 5], [5, 6, 7]])
+        lons2 = np.array([[4, 5, 6], [6, 7, 8]])
+        lats2 = np.array([[4, 5, 6], [6, 7, 8]])
+        swath_def1 = geometry.SwathDefinition(lons1, lats1)
+        swath_def2 = geometry.SwathDefinition(lons2, lats2)
+        swath_def_concat = swath_def1.concatenate(swath_def2)
+        expected = np.array(
+            [[1, 2, 3], [3, 4, 5], [5, 6, 7], [4, 5, 6], [6, 7, 8]])
+        self.assertTrue(np.array_equal(swath_def_concat.lons, expected) and
+                        np.array_equal(swath_def_concat.lons, expected),
+                        'Failed to concatenate 2D swaths')
+
+    def test_append_1d(self):
+        lons1 = np.array([1, 2, 3])
+        lats1 = np.array([1, 2, 3])
+        lons2 = np.array([4, 5, 6])
+        lats2 = np.array([4, 5, 6])
+        swath_def1 = geometry.SwathDefinition(lons1, lats1)
+        swath_def2 = geometry.SwathDefinition(lons2, lats2)
+        swath_def1.append(swath_def2)
+        expected = np.array([1, 2, 3, 4, 5, 6])
+        self.assertTrue(np.array_equal(swath_def1.lons, expected) and
+                        np.array_equal(swath_def1.lons, expected),
+                        'Failed to append 1D swaths')
+
+    def test_append_2d(self):
+        lons1 = np.array([[1, 2, 3], [3, 4, 5], [5, 6, 7]])
+        lats1 = np.array([[1, 2, 3], [3, 4, 5], [5, 6, 7]])
+        lons2 = np.array([[4, 5, 6], [6, 7, 8]])
+        lats2 = np.array([[4, 5, 6], [6, 7, 8]])
+        swath_def1 = geometry.SwathDefinition(lons1, lats1)
+        swath_def2 = geometry.SwathDefinition(lons2, lats2)
+        swath_def1.append(swath_def2)
+        expected = np.array(
+            [[1, 2, 3], [3, 4, 5], [5, 6, 7], [4, 5, 6], [6, 7, 8]])
+        self.assertTrue(np.array_equal(swath_def1.lons, expected) and
+                        np.array_equal(swath_def1.lons, expected),
+                        'Failed to append 2D swaths')
+
+    def test_swath_equal(self):
+        """Test swath equality."""
+        lons = np.array([1.2, 1.3, 1.4, 1.5])
+        lats = np.array([65.9, 65.86, 65.82, 65.78])
+        swath_def = geometry.SwathDefinition(lons, lats)
+        swath_def2 = geometry.SwathDefinition(lons, lats)
+        self.assertFalse(
+            swath_def != swath_def2, 'swath_defs are not equal as expected')
+
+    def test_swath_not_equal(self):
+        """Test swath inequality."""
+        lats1 = np.array([65.9, 65.86, 65.82, 65.78])
+        lons = np.array([1.2, 1.3, 1.4, 1.5])
+        lats2 = np.array([65.91, 65.85, 65.80, 65.75])
+        swath_def = geometry.SwathDefinition(lons, lats1)
+        swath_def2 = geometry.SwathDefinition(lons, lats2)
+        self.assertFalse(
+            swath_def == swath_def2, 'swath_defs are not expected to be equal')
+
+    def test_compute_omerc_params(self):
+        """Test omerc parameters computation."""
+        lats = np.array([[85.23900604248047, 62.256004333496094, 35.58000183105469],
+                         [80.84000396728516, 60.74200439453125, 34.08500289916992],
+                         [67.07600402832031, 54.147003173828125, 30.547000885009766]]).T
+
+        lons = np.array([[-90.67900085449219, -21.565000534057617, -21.525001525878906],
+                         [79.11000061035156, 7.284000396728516, -5.107000350952148],
+                         [81.26400756835938, 29.672000885009766, 10.260000228881836]]).T
+
+        area = geometry.SwathDefinition(lons, lats)
+        proj_dict = {'no_rot': True, 'lonc': 5.340645620216994,
+                     'ellps': 'WGS84', 'proj': 'omerc',
+                     'alpha': 19.022450179020247, 'lat_0': 60.7420043944989}
+        assert_np_dict_allclose(area._compute_omerc_parameters('WGS84'),
+                                proj_dict)
+
+    def test_get_edge_lonlats(self):
+        """Test the `get_edge_lonlats` functionality."""
+        lats = np.array([[85.23900604248047, 62.256004333496094, 35.58000183105469],
+                         [80.84000396728516, 60.74200439453125, 34.08500289916992],
+                         [67.07600402832031, 54.147003173828125, 30.547000885009766]]).T
+
+        lons = np.array([[-90.67900085449219, -21.565000534057617, -21.525001525878906],
+                         [79.11000061035156, 7.284000396728516, -5.107000350952148],
+                         [81.26400756835938, 29.672000885009766, 10.260000228881836]]).T
+
+        area = geometry.SwathDefinition(lons, lats)
+        lons, lats = area.get_edge_lonlats()
+
+        np.testing.assert_allclose(lons, [-90.67900085, 79.11000061,  81.26400757,
+                                          81.26400757, 29.67200089, 10.26000023,
+                                          10.26000023, -5.10700035, -21.52500153,
+                                          -21.52500153, -21.56500053, -90.67900085])
+        np.testing.assert_allclose(lats, [85.23900604, 80.84000397, 67.07600403,
+                                          67.07600403, 54.14700317, 30.54700089,
+                                          30.54700089, 34.0850029, 35.58000183,
+                                          35.58000183, 62.25600433,  85.23900604])
+
+        lats = np.array([[80., 80., 80.],
+                         [80., 90., 80],
+                         [80., 80., 80.]]).T
+
+        lons = np.array([[-45., 0., 45.],
+                         [-90, 0., 90.],
+                         [-135., -180., 135.]]).T
+
+        area = geometry.SwathDefinition(lons, lats)
+        lons, lats = area.get_edge_lonlats()
+
+        np.testing.assert_allclose(lons, [-45., -90., -135., -135., -180., 135.,
+                                          135., 90., 45., 45., 0., -45.])
+        np.testing.assert_allclose(lats, [80., 80., 80., 80., 80., 80., 80.,
+                                          80., 80., 80., 80., 80.])
+
+    def test_compute_optimal_bb(self):
+        """Test computing the bb area."""
+        lats = np.array([[85.23900604248047, 62.256004333496094, 35.58000183105469],
+                         [80.84000396728516, 60.74200439453125, 34.08500289916992],
+                         [67.07600402832031, 54.147003173828125, 30.547000885009766]]).T
+
+        lons = np.array([[-90.67900085449219, -21.565000534057617, -21.525001525878906],
+                         [79.11000061035156, 7.284000396728516, -5.107000350952148],
+                         [81.26400756835938, 29.672000885009766, 10.260000228881836]]).T
+
+        area = geometry.SwathDefinition(lons, lats)
+
+        res = area.compute_optimal_bb_area({'proj': 'omerc', 'ellps': 'WGS84'})
+
+        np.testing.assert_allclose(res.area_extent, (2286629.731529,
+                                                     -2359693.817959,
+                                                     11729881.856072,
+                                                     2437001.523925))
+        proj_dict = {'no_rot': True, 'lonc': 5.340645620216994,
+                     'ellps': 'WGS84', 'proj': 'omerc',
+                     'alpha': 19.022450179020247, 'lat_0': 60.7420043944989}
+        assert_np_dict_allclose(res.proj_dict, proj_dict)
+        self.assertEqual(res.shape, (3, 3))
+
+
 class TestStackedAreaDefinition(unittest.TestCase):
     """Test the StackedAreaDefition."""
 
@@ -859,6 +1035,86 @@ class TestStackedAreaDefinition(unittest.TestCase):
                                      area_extent)
 
 
+class TestDynamicAreaDefinition(unittest.TestCase):
+    """Test the DynamicAreaDefinition class."""
+
+    def test_freeze(self):
+        """Test freezing the area."""
+        area = geometry.DynamicAreaDefinition('test_area', 'A test area',
+                                              {'proj': 'laea'})
+        lons = [10, 10, 22, 22]
+        lats = [50, 66, 66, 50]
+        result = area.freeze((lons, lats),
+                             resolution=3000,
+                             proj_info={'lon0': 16, 'lat0': 58})
+
+        np.testing.assert_allclose(result.area_extent, (538546.7274949469,
+                                                        5380808.879250369,
+                                                        1724415.6519203288,
+                                                        6998895.701001488))
+        self.assertEqual(result.proj_dict['lon0'], 16)
+        self.assertEqual(result.proj_dict['lat0'], 58)
+        self.assertEqual(result.x_size, 395)
+        self.assertEqual(result.y_size, 539)
+
+    def test_freeze_with_bb(self):
+        """Test freezing the area with bounding box computation."""
+        # area = geometry.DynamicAreaDefinition('test_area', 'A test area',
+        #                                       {'proj': 'omerc'},
+        #                                       optimize_projection=False)
+        # lons = [[10, 12.1, 14.2, 16.3],
+        #         [10, 12, 14, 16],
+        #         [10, 11.9, 13.8, 15.7]]
+        # lats = [[66, 67, 68, 69.],
+        #         [58, 59, 60, 61],
+        #         [50, 51, 52, 53]]
+        # sdef = geometry.SwathDefinition(lons, lats)
+        # result = area.freeze(sdef,
+        #                      resolution=1000)
+        # self.assertTupleEqual(result.area_extent, (5578795.1654752363,
+        #                                            -270848.61872542271,
+        #                                            7694893.3964453982,
+        #                                            126974.877141819))
+        # self.assertEqual(result.x_size, 2116)
+        # self.assertEqual(result.y_size, 398)
+
+        area = geometry.DynamicAreaDefinition('test_area', 'A test area',
+                                              {'proj': 'omerc'},
+                                              optimize_projection=True)
+        lons = [[10, 12.1, 14.2, 16.3],
+                [10, 12, 14, 16],
+                [10, 11.9, 13.8, 15.7]]
+        lats = [[66, 67, 68, 69.],
+                [58, 59, 60, 61],
+                [50, 51, 52, 53]]
+        sdef = geometry.SwathDefinition(lons, lats)
+        result = area.freeze(sdef,
+                             resolution=1000)
+        np.testing.assert_allclose(result.area_extent, (5050520.6077326955,
+                                                        -336485.86803662963,
+                                                        8223167.9541879389,
+                                                        192612.12645302597))
+        self.assertEqual(result.x_size, 3)
+        self.assertEqual(result.y_size, 4)
+
+    def test_compute_domain(self):
+        """Test computing size and area extent."""
+        area = geometry.DynamicAreaDefinition('test_area', 'A test area',
+                                              {'proj': 'laea'})
+        corners = [1, 1, 9, 9]
+        self.assertRaises(ValueError, area.compute_domain, corners, 1, 1)
+
+        area_extent, x_size, y_size = area.compute_domain(corners, size=(5, 5))
+        self.assertTupleEqual(area_extent, (0, 0, 10, 10))
+        self.assertEqual(x_size, 5)
+        self.assertEqual(y_size, 5)
+
+        area_extent, x_size, y_size = area.compute_domain(corners, resolution=2)
+        self.assertTupleEqual(area_extent, (0, 0, 10, 10))
+        self.assertEqual(x_size, 5)
+        self.assertEqual(y_size, 5)
+
+
 def suite():
     """The test suite.
     """
@@ -866,6 +1122,8 @@ def suite():
     mysuite = unittest.TestSuite()
     mysuite.addTest(loader.loadTestsFromTestCase(Test))
     mysuite.addTest(loader.loadTestsFromTestCase(TestStackedAreaDefinition))
+    mysuite.addTest(loader.loadTestsFromTestCase(TestDynamicAreaDefinition))
+    mysuite.addTest(loader.loadTestsFromTestCase(TestSwathDefinition))
 
     return mysuite
 
